using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.Window;

namespace SlatedGameToolkit.Framework.Graphics.OpenGL
{
    public abstract class GLProgram : IDisposable
    {
        private WindowContext context;
        private protected UIntPtr handle;
        private protected GLCreateProgram createProgram;
        private protected GLDeleteProgram deleteProgram;
        private protected GLLinkProgram linkProgram;
        private protected GLGetProgramInfoLog getProgramInfoLog;

        public GLProgram(WindowContext context) {
            this.context = context;
            context.MakeCurrent();
            createProgram = Marshal.GetDelegateForFunctionPointer<GLCreateProgram>(SDL.SDL_GL_GetProcAddress("glCreateProgram"));
            if (createProgram == null) throw new FrameworkSDLException();
            deleteProgram = Marshal.GetDelegateForFunctionPointer<GLDeleteProgram>(SDL.SDL_GL_GetProcAddress("glDeleteProgram"));
            if (deleteProgram == null) throw new FrameworkSDLException();
            linkProgram = Marshal.GetDelegateForFunctionPointer<GLLinkProgram>(SDL.SDL_GL_GetProcAddress("glLinkProgram"));
            if (linkProgram == null) throw new FrameworkSDLException();
            getProgramInfoLog = Marshal.GetDelegateForFunctionPointer<GLGetProgramInfoLog>(SDL.SDL_GL_GetProcAddress("glGetProgramInfoLog"));
            if (getProgramInfoLog == null) throw new FrameworkSDLException();

            handle = createProgram();
        }

        public virtual void Dispose()
        {
            deleteProgram(handle);
        }

        ~GLProgram() {
            Dispose();
        }
    }
}